import { NonterminalKind } from '@nomicfoundation/slang/kinds/index.js';
import { TerminalNode } from '@nomicfoundation/slang/cst/index.js';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { VersionRange } from './VersionRange.js';
import { VersionComparator } from './VersionComparator.js';
import { VersionSpecifiers } from './VersionSpecifiers.js';
export class VersionExpression {
    constructor(ast, offset) {
        this.kind = NonterminalKind.VersionExpression;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        if (ast.variant instanceof TerminalNode) {
            // TODO: test whether this is a Identifier
            this.variant = ast.variant.text;
        }
        else {
            switch (ast.variant.cst.kind) {
                case NonterminalKind.VersionRange:
                    this.variant = new VersionRange(ast.variant, offsets[0]);
                    break;
                case NonterminalKind.VersionComparator:
                    this.variant = new VersionComparator(ast.variant, offsets[0]);
                    break;
                case NonterminalKind.VersionSpecifiers:
                    this.variant = new VersionSpecifiers(ast.variant, offsets[0]);
                    break;
                default:
                    throw new Error(`Unexpected variant: ${ast.variant.cst.kind}`);
            }
        }
        metadata = updateMetadata(metadata, typeof this.variant === 'string' ? [] : [this.variant]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return typeof this.variant === 'string'
            ? this.variant
            : path.call(print, 'variant');
    }
}
//# sourceMappingURL=VersionExpression.js.map